@@ -4,6 +4,7 @@
 #include <list>
 #include <math.h>
 #include <stdio.h>
+#include <R.h> // for Rprintf
 
 #define NEWTON_EPSILON 1e-12
 #define NEWTON_STEPS 100
@@ -106,9 +107,9 @@ double PoissonLossPieceLog::get_larger_root(double equals){
        closest_negative_mean = candidate_root;
      }
      if(NEWTON_STEPS <= ++step){
-       // printf("larger root MAXSTEPS with equals=%e\n", equals);
+       // Rprintf("larger root MAXSTEPS with equals=%e\n", equals);
        // print();
-       // printf("step=%d mean=%e cost=%e\n", step, candidate_root, candidate_cost);
+       // Rprintf("step=%d mean=%e cost=%e\n", step, candidate_root, candidate_cost);
        return log((closest_positive_mean + closest_negative_mean)/2);
      }
      deriv = PoissonDeriv(candidate_root);
@@ -117,13 +118,13 @@ double PoissonLossPieceLog::get_larger_root(double equals){
        //it overshot to the left of the optimum, so the root is
        //probably very close to the optimum, and we have probably
        //already explored very close to the zero.
-       printf("larger root WRONG SIDE equals=%e\n", equals);
+       Rprintf("larger root WRONG SIDE equals=%e\n", equals);
        print();
-       printf("neg_cost=%e neg_mean=%e pos_cost=%e pos_mean=%e\n", closest_negative_cost, closest_negative_mean, closest_positive_cost, closest_positive_mean);
+       Rprintf("neg_cost=%e neg_mean=%e pos_cost=%e pos_mean=%e\n", closest_negative_cost, closest_negative_mean, closest_positive_cost, closest_positive_mean);
        if(closest_negative_cost==-INFINITY){
 	 double optimal_log_mean = argmin(); //min or max!
 	 double optimal_cost2 = getCost(optimal_log_mean);
-	 printf("optimal_mean=%e=%e=exp(%e) optimal_cost=%e=%e=\n", optimal_mean, exp(optimal_log_mean), optimal_log_mean, optimal_cost, optimal_cost2);
+	 Rprintf("optimal_mean=%e=%e=exp(%e) optimal_cost=%e=%e=\n", optimal_mean, exp(optimal_log_mean), optimal_log_mean, optimal_cost, optimal_cost2);
 	 throw 1;
        }
        return log((closest_positive_mean + closest_negative_mean)/2);
@@ -133,7 +134,7 @@ double PoissonLossPieceLog::get_larger_root(double equals){
        candidate_root = possibly_outside;
      }
   }while(NEWTON_EPSILON < ABS(candidate_cost));
-  //printf("found root %e in %d steps!\n", candidate_root, step);
+  //Rprintf("found root %e in %d steps!\n", candidate_root, step);
   return log(candidate_root);
 }
 
@@ -180,9 +181,9 @@ double PoissonLossPieceLog::get_smaller_root(double equals){
        closest_negative_log_mean = candidate_root;
      }
      if(NEWTON_STEPS <= ++step){
-       // printf("smaller root MAXSTEPS equals=%e\n", equals);
+       // Rprintf("smaller root MAXSTEPS equals=%e\n", equals);
        // print();
-       // printf("step=%d log_mean=%e cost=%e\n", step, candidate_root, candidate_cost);
+       // Rprintf("step=%d log_mean=%e cost=%e\n", step, candidate_root, candidate_cost);
        return (closest_positive_log_mean + closest_negative_log_mean)/2;
      }
      deriv = getDeriv(candidate_root);
@@ -194,9 +195,9 @@ double PoissonLossPieceLog::get_smaller_root(double equals){
        // it's on the right of the optimum, so the root is probably
        //very close to the optimum, and we have probably already
        //explored very close to the zero.
-       printf("smaller root WRONG SIDE equals=%e\n", equals);
+       Rprintf("smaller root WRONG SIDE equals=%e\n", equals);
        print();
-       printf("neg_cost=%e neg_log_mean=%e pos_cost=%e pos_log_mean=%e\n", closest_negative_cost, closest_negative_log_mean, closest_positive_cost, closest_positive_log_mean);
+       Rprintf("neg_cost=%e neg_log_mean=%e pos_cost=%e pos_log_mean=%e\n", closest_negative_cost, closest_negative_log_mean, closest_positive_cost, closest_positive_log_mean);
        return (closest_positive_log_mean + closest_negative_log_mean)/2;
      }
   }while(NEWTON_EPSILON < ABS(candidate_cost));
@@ -265,11 +266,11 @@ void PiecewisePoissonLossLog::set_to_min_less_of
   while(it != input->piece_list.end()){
     double left_cost = it->getCost(it->min_log_mean);
     double right_cost = it->getCost(it->max_log_mean);
-    if(verbose)printf("left_cost=%f right_cost=%f\n", left_cost, right_cost);
+    if(verbose)Rprintf("left_cost=%f right_cost=%f\n", left_cost, right_cost);
     if(prev_min_cost == INFINITY){
       // Look for min achieved in this interval.
       if(verbose){
-	printf("Searching for min in\n");
+	Rprintf("Searching for min in\n");
 	it->print();
       }
       double next_left_cost;
@@ -280,14 +281,14 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	// negative, we know that this function must be increasing or
 	// numerically constant on this interval.
 	// g(x) = Linear*e^x + Constant,
-	if(verbose)printf("DEGENERATE LINEAR FUNCTION IN MIN LESS\n");
+	if(verbose)Rprintf("DEGENERATE LINEAR FUNCTION IN MIN LESS\n");
 	// We used to check if(it->Linear==0) but there are some cases
 	// when the function has a non-zero Linear coefficient, but is
 	// numerically constant (e.g. Linear=156 between -inf and
 	// -44). So now we check to see if the cost on the left and
 	// right of the interval are equal.
 	double right_left_diff = right_cost - left_cost;
-	if(verbose)printf("right_cost-left_cost=%e\n", right_left_diff);
+	if(verbose)Rprintf("right_cost-left_cost=%e\n", right_left_diff);
 	bool right_left_equal = right_left_diff < NEWTON_EPSILON;
 	bool next_cost_more_than_left;
 	if(next_it == input->piece_list.end()){
@@ -295,7 +296,7 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	}else{
 	  next_left_cost = next_it->getCost(next_it->min_log_mean);
 	  double next_left_diff = next_left_cost-left_cost;
-	  if(verbose)printf("next_left_cost-left_cost=%e\n", next_left_diff);
+	  if(verbose)Rprintf("next_left_cost-left_cost=%e\n", next_left_diff);
 	  next_cost_more_than_left = NEWTON_EPSILON < next_left_diff;
 	}
 	// next_cost_more_than_left is true if the cost on the left of
@@ -309,12 +310,12 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	  prev_min_cost = left_cost;
 	  prev_best_log_mean = it->min_log_mean;
 	  if(verbose){
-	    printf("Increasing interval left_cost=%e(stored) right_cost=%e diff=%e\n", left_cost, right_cost, right_cost-left_cost);
+	    Rprintf("Increasing interval left_cost=%e(stored) right_cost=%e diff=%e\n", left_cost, right_cost, right_cost-left_cost);
 	    it->print();
 	  }
 	}else{
 	  if(verbose){
-	    printf("Numerically constant convex piece\n");
+	    Rprintf("Numerically constant convex piece\n");
 	    it->print();
 	  }
 	  // store this numerically constant interval.
@@ -339,8 +340,8 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	// next function piece. This is necessary because sometimes
 	// there are numerical issues.
  	if(verbose){
-	  printf("min cost=%f at log_mean=%f\n", mu_cost, mu);
-	  printf("next_left_cost-mu_cost=%e right_cost-mu_cost=%e\n", next_left_cost-mu_cost, right_cost-mu_cost);
+	  Rprintf("min cost=%f at log_mean=%f\n", mu_cost, mu);
+	  Rprintf("next_left_cost-mu_cost=%e right_cost-mu_cost=%e\n", next_left_cost-mu_cost, right_cost-mu_cost);
 	}
 	bool cost_ok = NEWTON_EPSILON < right_cost-mu_cost && next_ok;
 	if(mu <= it->min_log_mean && cost_ok){
@@ -349,7 +350,7 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	     interval. We don't need to store it, but we do need to keep
 	     track of the minimum cost, which occurs at the min mean
 	     value in this interval. */
-	  if(verbose)printf("min before interval\n");
+	  if(verbose)Rprintf("min before interval\n");
 	  prev_min_cost = it->getCost(it->min_log_mean);
 	  prev_best_log_mean = it->min_log_mean;
 	}else if(mu < it->max_log_mean && cost_ok){
@@ -358,9 +359,9 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	  // piece later. NB it is possible that prev_min_log_mean==mu in
 	  // which case we do not need to store the convex piece.
 	  if(verbose){
-	    printf("min in this interval at log_mean=%f cost=%f\n", mu, mu_cost);
-	    printf("right_cost=%f right-constant=%e\n", right_cost, right_cost-mu_cost);
-	    printf("next_left_cost=%f next-constant=%e\n", next_left_cost, next_left_cost-mu_cost);
+	    Rprintf("min in this interval at log_mean=%f cost=%f\n", mu, mu_cost);
+	    Rprintf("right_cost=%f right-constant=%e\n", right_cost, right_cost-mu_cost);
+	    Rprintf("next_left_cost=%f next-constant=%e\n", next_left_cost, next_left_cost-mu_cost);
 	  }
 	  if(prev_min_log_mean < mu){
 	    piece_list.emplace_back
@@ -370,12 +371,12 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	  prev_min_log_mean = mu;
 	  prev_best_log_mean = mu;
 	  prev_min_cost = mu_cost;
-	  if(verbose)printf("prev_min_cost=%f\n", prev_min_cost);
+	  if(verbose)Rprintf("prev_min_cost=%f\n", prev_min_cost);
 	}else{
 	  // Minimum after this interval, so this function is
 	  // decreasing on this entire interval, and so we can just
 	  // store it as is.
-	  if(verbose)printf("min after interval\n");
+	  if(verbose)Rprintf("min after interval\n");
 	  piece_list.emplace_back
 	    (it->Linear, it->Log, it->Constant, prev_min_log_mean, it->max_log_mean,
 	     PREV_NOT_SET, INFINITY); // equality constraint active on convex piece.
@@ -385,8 +386,8 @@ void PiecewisePoissonLossLog::set_to_min_less_of
     }else{//prev_min_cost is finite
       // Look for a function with prev_min_cost in its interval.
       if(verbose){
-	printf("Searching for intersection with %f\n", prev_min_cost);
-	printf("cost at limits=[%f,%f] cost-constant=[%e,%e]\n",
+	Rprintf("Searching for intersection with %f\n", prev_min_cost);
+	Rprintf("cost at limits=[%f,%f] cost-constant=[%e,%e]\n",
 	       left_cost, right_cost,
 	       left_cost-prev_min_cost, right_cost-prev_min_cost);
 	it->print();
@@ -425,7 +426,7 @@ void PiecewisePoissonLossLog::set_to_min_less_of
 	}//if(has two roots
 	if(right_cost <= prev_min_cost+NEWTON_EPSILON && prev_min_cost < INFINITY){
 	  //ends exactly/numerically on the right.
-	  if(verbose)printf("constant numerically equal on right\n");
+	  if(verbose)Rprintf("constant numerically equal on right\n");
 	  piece_list.emplace_back
 	    (0, 0, prev_min_cost,
 	     prev_min_log_mean, it->max_log_mean, 
@@ -438,7 +439,7 @@ void PiecewisePoissonLossLog::set_to_min_less_of
     }//if(prev_min_cost is finite
     it++;
     if(verbose){
-      printf("current min-less-------------------\n");
+      Rprintf("current min-less-------------------\n");
       print();
     }
   }//while(it
@@ -468,11 +469,11 @@ void PiecewisePoissonLossLog::set_to_min_more_of
     if(prev_min_cost == INFINITY){
       // Look for min achieved in this interval.
       if(verbose){
-	printf("Searching for min in\n");
+	Rprintf("Searching for min in\n");
 	it->print();
       }
       if(it->Log==0){
-	if(verbose)printf("DEGENERATE LINEAR FUNCTION IN MIN MORE\n");
+	if(verbose)Rprintf("DEGENERATE LINEAR FUNCTION IN MIN MORE\n");
 	//degenerate Linear function. since the Linear coef is never
 	//negative, we know that this function must be increasing or
 	//numerically constant on this interval. In both cases we
@@ -501,7 +502,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
 	  /* The minimum is achieved after this interval, so this
 	     function is always decreasing in this interval. We don't
 	     need to store it. */
-	  if(verbose)printf("min after this interval\n");
+	  if(verbose)Rprintf("min after this interval\n");
 	  prev_min_cost = it->getCost(it->max_log_mean);
 	  prev_best_log_mean = it->max_log_mean;
 	}else if(it->min_log_mean < mu && NEWTON_EPSILON < this_cost_left-mu_cost && prev_ok){
@@ -509,7 +510,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
 	  // min, and keep track of the min cost to create a constant
 	  // piece later. NB it is possible that mu==prev_max_log_mean, in
 	  // which case we do not need to save the convex piece.
-	  if(verbose)printf("min in this interval at mu=%f\n", mu);
+	  if(verbose)Rprintf("min in this interval at mu=%f\n", mu);
 	  if(mu < prev_max_log_mean){ 
 	    piece_list.emplace_front
 	      (it->Linear, it->Log, it->Constant, mu, prev_max_log_mean, 
@@ -522,7 +523,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
 	  // Minimum before this interval, so this function is
 	  // increasing on this entire interval, and so we can just
 	  // store it as is.
-	  if(verbose)printf("min before this interval\n");
+	  if(verbose)Rprintf("min before this interval\n");
 	  piece_list.emplace_front
 	    (it->Linear, it->Log, it->Constant, it->min_log_mean, prev_max_log_mean,
 	     PREV_NOT_SET, INFINITY); // equality constraint active on convex piece.
@@ -534,8 +535,8 @@ void PiecewisePoissonLossLog::set_to_min_more_of
       double left_cost = it->getCost(it->min_log_mean);
       double right_cost = it->getCost(it->max_log_mean);
       if(verbose){
-	printf("Searching for intersection with %f\n", prev_min_cost);
-	printf("cost at limits=[%f,%f] cost-constant=[%e,%e]\n",
+	Rprintf("Searching for intersection with %f\n", prev_min_cost);
+	Rprintf("cost at limits=[%f,%f] cost-constant=[%e,%e]\n",
 	       left_cost, right_cost,
 	       left_cost-prev_min_cost, right_cost-prev_min_cost);
 	it->print();
@@ -544,7 +545,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
       if(it->Log==0){
 	//degenerate Linear case, there is one intersection point.
 	mu = log((prev_min_cost - it->Constant)/it->Linear);
-	if(verbose)printf("degenerate linear intersection at log_mean=%f\n", mu);
+	if(verbose)Rprintf("degenerate linear intersection at log_mean=%f\n", mu);
       }else{// Log is not zero.
 	// Theoretically there can be zero, one, or two intersection
 	// points between the constant function prev_min_log_mean and a
@@ -555,14 +556,14 @@ void PiecewisePoissonLossLog::set_to_min_more_of
 	  // are only concerned with the second mean value (the
 	  // greater of the two). 
 	  mu = it->get_larger_root(prev_min_cost);
-	  if(verbose)printf("large root log_mean=%f\n", mu);
+	  if(verbose)Rprintf("large root log_mean=%f\n", mu);
 	}//if(there are two roots
       }//if(Log is zero
       if(it->min_log_mean < mu && mu < it->max_log_mean){
 	// The intersection point occurs within the interval, so the
 	// constant interval ends here, and we can store it
 	// immediately.
-	if(verbose)printf("%f in interval\n", mu);
+	if(verbose)Rprintf("%f in interval\n", mu);
 	piece_list.emplace_front
 	  (0, 0, prev_min_cost,
 	   mu, prev_max_log_mean,
@@ -573,7 +574,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
 	it++;
       }else if(left_cost <= prev_min_cost+NEWTON_EPSILON){
 	//ends exactly/numerically on the left.
-	if(verbose)printf("constant numerically equal on left\n");
+	if(verbose)Rprintf("constant numerically equal on left\n");
 	piece_list.emplace_front
 	  (0, 0, prev_min_cost,
 	   it->min_log_mean, prev_max_log_mean,
@@ -584,7 +585,7 @@ void PiecewisePoissonLossLog::set_to_min_more_of
       }
     }//if(prev_min_cost is finite
     if(verbose){
-      printf("current min-more-------------------\n");
+      Rprintf("current min-more-------------------\n");
       print();
     }
   }//while(it
@@ -648,7 +649,7 @@ double PiecewisePoissonLossLog::findCost(double mean){
 
 void PiecewisePoissonLossLog::print(){
   PoissonLossPieceListLog::iterator it;
-  printf("%10s %10s %15s %15s %15s %15s %s\n",
+  Rprintf("%10s %10s %15s %15s %15s %15s %s\n",
 	 "Linear", "Log", "Constant",
 	 "min_log_mean", "max_log_mean",
 	 "prev_log_mean", "data_i");
@@ -658,7 +659,7 @@ void PiecewisePoissonLossLog::print(){
 }
 
 void PoissonLossPieceLog::print(){
-  printf("%.20e %.20e %.20e %15f %15f %15f %d\n",
+  Rprintf("%.20e %.20e %.20e %15f %15f %15f %d\n",
 	 Linear, Log, Constant,
 	 min_log_mean, max_log_mean,
 	 prev_log_mean, data_i);
@@ -700,20 +701,20 @@ int PiecewisePoissonLossLog::check_min_of
       PoissonLossPieceListLog::iterator pit = it;
       pit--;
       if(pit->max_log_mean != it->min_log_mean){
-	printf("prev->max_log_mean != it->min_log_mean min\n");
+	Rprintf("prev->max_log_mean != it->min_log_mean min\n");
 	return 3;
       }
       // double cost_prev = pit->getCost(pit->max_log_mean);
       // double cost_here = it->getCost(it->min_log_mean);
       // if(0.01 < ABS(cost_prev - cost_here)){
-      // 	printf("discontinuity detected at %f, %f != %f\n", pit->max_log_mean, cost_prev, cost_here);
+      // 	Rprintf("discontinuity detected at %f, %f != %f\n", pit->max_log_mean, cost_prev, cost_here);
       // 	pit->print();
       // 	it->print();
       // 	return 4;
       // }
     }
     if(it->max_log_mean <= it->min_log_mean){
-      printf("max_log_mean<=min_log_mean=%15.10f min\n", it->min_log_mean);
+      Rprintf("max_log_mean<=min_log_mean=%15.10f min\n", it->min_log_mean);
       return 2;
     }
     double mid_mean = (it->min_log_mean + it->max_log_mean)/2;
@@ -721,17 +722,17 @@ int PiecewisePoissonLossLog::check_min_of
       double cost_min = it->getCost(mid_mean);
       double cost_prev = prev->findCost(mid_mean);
       if(cost_prev+1e-6 < cost_min){
-	printf("prev(%f)=%f\n", mid_mean, cost_prev);
+	Rprintf("prev(%f)=%f\n", mid_mean, cost_prev);
 	prev->print();
-	printf("min(%f)=%f\n", mid_mean, cost_min);
+	Rprintf("min(%f)=%f\n", mid_mean, cost_min);
 	print();
 	return 1;
       }
       double cost_model = model->findCost(mid_mean);
       if(cost_model+1e-6 < cost_min){
-	printf("model(%f)=%f\n", mid_mean, cost_model);
+	Rprintf("model(%f)=%f\n", mid_mean, cost_model);
 	model->print();
-	printf("min(%f)=%f\n", mid_mean, cost_min);
+	Rprintf("min(%f)=%f\n", mid_mean, cost_min);
 	print();
 	return 1;
       }
@@ -742,12 +743,12 @@ int PiecewisePoissonLossLog::check_min_of
       PoissonLossPieceListLog::iterator pit = it;
       pit--;
       if(pit->max_log_mean != it->min_log_mean){
-	printf("prev->max_log_mean != it->min_log_mean prev\n");
+	Rprintf("prev->max_log_mean != it->min_log_mean prev\n");
 	return 3;
       }
     }
     if(it->max_log_mean <= it->min_log_mean){
-      printf("max_log_mean<=min_log_mean=%15.10f prev\n", it->min_log_mean);
+      Rprintf("max_log_mean<=min_log_mean=%15.10f prev\n", it->min_log_mean);
       return 2;
     }
     double mid_mean = (it->min_log_mean + it->max_log_mean)/2;
@@ -755,9 +756,9 @@ int PiecewisePoissonLossLog::check_min_of
       double cost_prev = it->getCost(mid_mean);
       double cost_min = findCost(mid_mean);
       if(cost_prev+1e-6 < cost_min){
-	printf("prev(%f)=%f\n", mid_mean, cost_prev);
+	Rprintf("prev(%f)=%f\n", mid_mean, cost_prev);
 	prev->print();
-	printf("min(%f)=%f\n", mid_mean, cost_min);
+	Rprintf("min(%f)=%f\n", mid_mean, cost_min);
 	print();
 	return 1;
       }
@@ -768,12 +769,12 @@ int PiecewisePoissonLossLog::check_min_of
       PoissonLossPieceListLog::iterator pit = it;
       pit--;
       if(pit->max_log_mean != it->min_log_mean){
-	printf("prev->max_log_mean != it->min_log_mean model\n");
+	Rprintf("prev->max_log_mean != it->min_log_mean model\n");
 	return 3;
       }
     }
     if(it->max_log_mean <= it->min_log_mean){
-      printf("max_log_mean<=min_log_mean=%15.10f model\n", it->min_log_mean);
+      Rprintf("max_log_mean<=min_log_mean=%15.10f model\n", it->min_log_mean);
       return 2;
     }
     double mid_mean = (it->min_log_mean + it->max_log_mean)/2;
@@ -781,9 +782,9 @@ int PiecewisePoissonLossLog::check_min_of
       double cost_model = it->getCost(mid_mean);
       double cost_min = findCost(mid_mean);
       if(cost_model+1e-6 < cost_min){
-	printf("model(%f)=%f\n", mid_mean, cost_model);
+	Rprintf("model(%f)=%f\n", mid_mean, cost_model);
 	model->print();
-	printf("min(%f)=%f\n", mid_mean, cost_min);
+	Rprintf("min(%f)=%f\n", mid_mean, cost_min);
 	print();
 	return 1;
       }
@@ -798,10 +799,10 @@ void PiecewisePoissonLossLog::set_to_min_env_of
     it1 = fun1->piece_list.begin(),
     it2 = fun2->piece_list.begin();
   if(verbose){
-    printf("computing min env of:\n");
-    printf("=min-less/more\n");
+    Rprintf("computing min env of:\n");
+    Rprintf("=min-less/more\n");
     fun1->print();
-    printf("=cost model\n");
+    Rprintf("=cost model\n");
     fun2->print();
   }
   piece_list.clear();
@@ -810,7 +811,7 @@ void PiecewisePoissonLossLog::set_to_min_env_of
     push_min_pieces(fun1, fun2, it1, it2, verbose);
     if(verbose){
       print();
-      printf("------\n");
+      Rprintf("------\n");
     }
     double last_max_log_mean = piece_list.back().max_log_mean;
     if(it1->max_log_mean == last_max_log_mean){
@@ -869,23 +870,23 @@ void PiecewisePoissonLossLog::push_min_pieces
   bool same_at_right;
   double first_max_log_mean;
   if(it1->max_log_mean < it2->max_log_mean){
-    if(verbose)printf("it2 function piece continues to the right of it1.\n");
+    if(verbose)Rprintf("it2 function piece continues to the right of it1.\n");
     same_at_right = sameFuns(next1, it2);
     first_max_log_mean = it1->max_log_mean;
   }else{
     first_max_log_mean = it2->max_log_mean;
     if(it2->max_log_mean < it1->max_log_mean){
-      if(verbose)printf("it2 function piece ends before it1.\n");
+      if(verbose)Rprintf("it2 function piece ends before it1.\n");
       same_at_right = sameFuns(it1, next2);
     }else{
-      if(verbose)printf("it2 and it1 end at same max_log_mean.\n");
+      if(verbose)Rprintf("it2 and it1 end at same max_log_mean.\n");
       if(next1==fun1->piece_list.end() &&
 	 next2==fun2->piece_list.end()){
-	if(verbose)printf("at the end so they can't be equal after this interval.\n");
+	if(verbose)Rprintf("at the end so they can't be equal after this interval.\n");
 	same_at_right = false;
       }else{
 	if(verbose){
-	  printf("comparing next function pieces.\n");
+	  Rprintf("comparing next function pieces.\n");
 	  next1->print();
 	  next2->print();
 	}
@@ -897,11 +898,11 @@ void PiecewisePoissonLossLog::push_min_pieces
     // we should probably never get here, but if we do, no need to
     // store this interval.
     if(verbose){
-      printf("prev\n");
+      Rprintf("prev\n");
       fun1->print();
-      printf("model\n");
+      Rprintf("model\n");
       fun2->print();
-      printf("interval size 0!-----------------\n");
+      Rprintf("interval size 0!-----------------\n");
     }
     return;
   }
@@ -909,7 +910,7 @@ void PiecewisePoissonLossLog::push_min_pieces
     // The functions are exactly equal over the entire interval so we
     // can push either of them.
     push_piece(it1, last_min_log_mean, first_max_log_mean);
-    if(verbose)printf("exactly equal over entire interval\n");
+    if(verbose)Rprintf("exactly equal over entire interval\n");
     return;
   }
   PoissonLossPieceLog diff_piece
@@ -924,7 +925,7 @@ void PiecewisePoissonLossLog::push_min_pieces
   double cost_diff_mid = diff_piece.getCost(log(mid_mean));
   // Easy case of equality on both left and right.
   if(same_at_left && same_at_right){
-    if(verbose)printf("Same on both the left and the right\n");
+    if(verbose)Rprintf("Same on both the left and the right\n");
     if(cost_diff_mid < 0){
       push_piece(it1, last_min_log_mean, first_max_log_mean);
     }else{
@@ -943,7 +944,7 @@ void PiecewisePoissonLossLog::push_min_pieces
       }else{
 	push_piece(it2, last_min_log_mean, first_max_log_mean);
       }
-      if(verbose)printf("offset by a constant=%e\n", diff_piece.Constant);
+      if(verbose)Rprintf("offset by a constant=%e\n", diff_piece.Constant);
       return;
     }
     if(diff_piece.Constant == 0){
@@ -953,7 +954,7 @@ void PiecewisePoissonLossLog::push_min_pieces
       }else{
 	push_piece(it2, last_min_log_mean, first_max_log_mean);
       }
-      if(verbose)printf("only diff is linear coef\n");
+      if(verbose)Rprintf("only diff is linear coef\n");
       return;
     }
     double log_mean_at_equal_cost = log(-diff_piece.Constant/diff_piece.Linear);
@@ -967,7 +968,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	push_piece(it2, last_min_log_mean, log_mean_at_equal_cost);
 	push_piece(it1, log_mean_at_equal_cost, first_max_log_mean);
       }
-      if(verbose)printf("Log zero with one root in interval\n");
+      if(verbose)Rprintf("Log zero with one root in interval\n");
       return;
     }
     // the root is outside the interval, so one is completely above
@@ -977,7 +978,7 @@ void PiecewisePoissonLossLog::push_min_pieces
     }else{
       push_piece(it2, last_min_log_mean, first_max_log_mean);
     }
-    if(verbose)printf("Log zero with no roots in interval\n");
+    if(verbose)Rprintf("Log zero with no roots in interval\n");
     return;
   }//if(diff->Log == 0
   double cost_diff_left = diff_piece.getCost(last_min_log_mean);
@@ -998,11 +999,11 @@ void PiecewisePoissonLossLog::push_min_pieces
       double cost_between_zeros = diff_piece.getCost(log_mean_between_zeros);
       double log_mean_at_optimum = diff_piece.argmin();
       if(verbose){
-	printf("cost_diff(left:%e)=%e\n", last_min_log_mean, cost_diff_left);
-	printf("cost_diff(cross:%e)=%e\n", log_mean_at_crossing, diff_piece.getCost(log_mean_at_crossing));
-	printf("cost_diff(between:%e)=%e\n", log_mean_between_zeros, cost_between_zeros);
-	printf("cost_diff(optimum:%e)=%e\n", log_mean_at_optimum, diff_piece.getCost(log_mean_at_optimum));
-	printf("cost_diff(right:%e)=%e\n", first_max_log_mean, cost_diff_right);
+	Rprintf("cost_diff(left:%e)=%e\n", last_min_log_mean, cost_diff_left);
+	Rprintf("cost_diff(cross:%e)=%e\n", log_mean_at_crossing, diff_piece.getCost(log_mean_at_crossing));
+	Rprintf("cost_diff(between:%e)=%e\n", log_mean_between_zeros, cost_between_zeros);
+	Rprintf("cost_diff(optimum:%e)=%e\n", log_mean_at_optimum, diff_piece.getCost(log_mean_at_optimum));
+	Rprintf("cost_diff(right:%e)=%e\n", first_max_log_mean, cost_diff_right);
       }
       if(last_min_log_mean < log_mean_at_crossing &&
 	 log_mean_at_crossing < log_mean_at_optimum &&
@@ -1015,7 +1016,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	  push_piece(it2, last_min_log_mean, log_mean_at_crossing);
 	  push_piece(it1, log_mean_at_crossing, first_max_log_mean);
 	}
-	if(verbose)printf("equal on the right with one crossing in interval\n");
+	if(verbose)Rprintf("equal on the right with one crossing in interval\n");
 	return;
       }
     }//if(two_roots
@@ -1026,7 +1027,7 @@ void PiecewisePoissonLossLog::push_min_pieces
     }else{
       push_piece(it2, last_min_log_mean, first_max_log_mean);
     }
-    if(verbose)printf("equal on the right with no crossing in interval\n");
+    if(verbose)Rprintf("equal on the right with no crossing in interval\n");
     return;
   }
   if(same_at_left){
@@ -1035,7 +1036,7 @@ void PiecewisePoissonLossLog::push_min_pieces
       // There could be a crossing point to the right.
       double log_mean_at_crossing = larger_log_mean;
       double log_mean_at_optimum = diff_piece.argmin();
-      if(verbose)printf("larger_log_mean=%f\n", log_mean_at_crossing);
+      if(verbose)Rprintf("larger_log_mean=%f\n", log_mean_at_crossing);
       if(last_min_log_mean < log_mean_at_optimum &&
 	 log_mean_at_optimum < log_mean_at_crossing &&
       	 log_mean_at_crossing < first_max_log_mean){
@@ -1047,7 +1048,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	  push_piece(it1, last_min_log_mean, log_mean_at_crossing);
 	  push_piece(it2, log_mean_at_crossing, first_max_log_mean);
 	}
-	if(verbose)printf("equal on the left with crossing in interval\n");
+	if(verbose)Rprintf("equal on the left with crossing in interval\n");
 	return;
       }
     }//if(there may be crossing
@@ -1056,7 +1057,7 @@ void PiecewisePoissonLossLog::push_min_pieces
     }else{
       push_piece(it2, last_min_log_mean, first_max_log_mean);
     }
-    if(verbose)printf("equal on the left with no crossing in interval\n");
+    if(verbose)Rprintf("equal on the left with no crossing in interval\n");
     return;
   }
   // The only remaining case is that the curves are equal neither on
@@ -1066,7 +1067,7 @@ void PiecewisePoissonLossLog::push_min_pieces
   if(two_roots){
     bool larger_inside =
       last_min_log_mean < larger_log_mean && larger_log_mean < first_max_log_mean;
-    if(verbose)printf("smaller_log_mean=%f %a\nlarger_log_mean=%f %a\n",
+    if(verbose)Rprintf("smaller_log_mean=%f %a\nlarger_log_mean=%f %a\n",
 		      smaller_log_mean, smaller_log_mean,
 		      larger_log_mean, larger_log_mean);
     bool smaller_inside =
@@ -1080,7 +1081,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	second_log_mean = larger_log_mean;
 	if(verbose){
 	  diff_piece.print();
-	  printf("%f and %f in [%f,%f]\n",
+	  Rprintf("%f and %f in [%f,%f]\n",
 		 smaller_log_mean, larger_log_mean,
 		 last_min_log_mean, first_max_log_mean);
 	}
@@ -1088,7 +1089,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	// smaller mean is not in the interval, but the larger is.
 	first_log_mean = larger_log_mean;
 	if(verbose){
-	  printf("%f in [%f,%f]\n",
+	  Rprintf("%f in [%f,%f]\n",
 		 first_log_mean,
 		 last_min_log_mean, first_max_log_mean);
 	}
@@ -1099,7 +1100,7 @@ void PiecewisePoissonLossLog::push_min_pieces
 	// smaller mean is in the interval, but not the larger.
 	first_log_mean = smaller_log_mean;
 	if(verbose){
-	  printf("%f in [%f,%f]\n",
+	  Rprintf("%f in [%f,%f]\n",
 		 first_log_mean,
 		 last_min_log_mean, first_max_log_mean);
 	}
@@ -1119,7 +1120,7 @@ void PiecewisePoissonLossLog::push_min_pieces
       push_piece(it1, first_log_mean, second_log_mean);
       push_piece(it2, second_log_mean, first_max_log_mean);
     }
-    if(verbose)printf("not equal on the sides, 2 crossing points\n");
+    if(verbose)Rprintf("not equal on the sides, 2 crossing points\n");
   }else if(first_log_mean != INFINITY){
     // "one" crossing point. actually sometimes we have last_min_log_mean
     // < first_log_mean < first_max_log_mean but cost_diff_before and
@@ -1128,11 +1129,11 @@ void PiecewisePoissonLossLog::push_min_pieces
     double before_mean = (exp(last_min_log_mean) + exp(first_log_mean))/2;
     double cost_diff_before = diff_piece.getCost(log(before_mean));
     if(verbose){
-      printf("cost_diff_before(%.55f)=%f\n", log(before_mean), cost_diff_before);
+      Rprintf("cost_diff_before(%.55f)=%f\n", log(before_mean), cost_diff_before);
     }
     double after_mean = (first_max_log_mean + first_log_mean)/2;
     double cost_diff_after = diff_piece.getCost(after_mean);
-    if(verbose)printf("cost_diff_after(%.55f)=%f\n", after_mean, cost_diff_after);
+    if(verbose)Rprintf("cost_diff_after(%.55f)=%f\n", after_mean, cost_diff_after);
     if(cost_diff_before < 0){
       if(cost_diff_after < 0){
 	// f1-f2<0 meaning f1<f2 on the entire interval, so just push it1.
@@ -1151,14 +1152,14 @@ void PiecewisePoissonLossLog::push_min_pieces
 	push_piece(it2, last_min_log_mean, first_max_log_mean);
       }
     }
-    if(verbose)printf("not equal on the sides, 1 crossing point\n");
+    if(verbose)Rprintf("not equal on the sides, 1 crossing point\n");
   }else{
     // "zero" crossing points. actually there may be a crossing point
     // in the interval that is numerically so close as to be identical
     // with last_min_log_mean or first_max_log_mean.
     if(verbose){
-      printf("not equal on the sides, zero crossing points\n");
-      printf("cost_diff left=%e mid=%e right=%e\n",
+      Rprintf("not equal on the sides, zero crossing points\n");
+      Rprintf("cost_diff left=%e mid=%e right=%e\n",
 	     cost_diff_left, cost_diff_mid, cost_diff_right);
     }
     double cost_diff;
